// Copyright (c) 2010 Keith D Gregory, all rights reserved
package com.kdgregory.app.s3util.dialogs;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.commons.lang.StringUtils;

import net.sf.swinglib.SwingUtil;
import net.sf.swinglib.UIHelper;
import net.sf.swinglib.actions.DialogCloseAction;
import net.sf.swinglib.field.FieldWatcher;

import com.kdgregory.app.s3util.main.Concierge;
import com.kdgregory.app.s3util.s3ops.S3BucketListOp;
import com.kdgregory.app.s3util.s3ops.S3InitialLoadOp;


/**
 *  Manages a modal dialog that allows the user to select/create a bucket.
 *  Once a bucket has been selected, the prefs are updated with that bucket's
 *  name and a request is sent to refresh the main window.
 */
public class BucketDialogController
implements S3BucketListOp.Callback
{
    private Concierge _concierge;

    // the dialog is lazily constructed by show()
    private FieldWatcher _fieldWatcher;
    private JDialog _dialog;
    private JList _fBucketList;
    private JTextField _fBucketName;


    public BucketDialogController(Concierge concierge)
    {
        _concierge = concierge;
    }


//----------------------------------------------------------------------------
//  Public Methods
//----------------------------------------------------------------------------

    /**
     *  Displays the (modal) dialog. Returns the selected bucket name, as a
     *  convenience for the caller.
     */
    public void show()
    {
        if (_dialog == null)
            constructDialog();

        _fieldWatcher.reset();
        refreshList();
        SwingUtil.center(_dialog, _concierge.getDialogOwner());
        _dialog.setVisible(true);
    }


//----------------------------------------------------------------------------
//  List updates
//----------------------------------------------------------------------------

    /**
     *  Resets the current list contents to "Loading" and kicks of a request
     *  for new contents.
     */
    private void refreshList()
    {
        _fBucketList.setListData(new String[] { "Loading..." });
        new S3BucketListOp(_concierge, this).start();
    }


    /**
     *  Called by the "list buckets" operation.
     */
    public void setBucketList(List<String> list)
    {
        String[] list2 = list.toArray(new String[list.size()]);
        _fBucketList.setListData(list2);

        String current = _concierge.getConfig().getAmazonBucketName();
        if (StringUtils.isNotEmpty(current))
        {
            _fBucketList.setSelectedValue(current, true);
            _fBucketName.setText(current);
        }
    }


//----------------------------------------------------------------------------
//  GUI construction / behavior
//----------------------------------------------------------------------------

    private void constructDialog()
    {
        OKAction okAction = new OKAction();
        DialogCloseAction cancelAction = new DialogCloseAction(null, "Cancel");

        _dialog = UIHelper.newModalDialog(
                    _concierge.getDialogOwner(),
                    "Create/Select Bucket",
                    createContentPane(),
                    okAction, cancelAction);
        cancelAction.setDialog(_dialog);

        _fieldWatcher = new FieldWatcher(okAction);
        _fieldWatcher.addWatchedField(_fBucketName);
    }


    private JPanel createContentPane()
    {
        _fBucketList = new JList();
        _fBucketList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        _fBucketList.getSelectionModel()
                    .addListSelectionListener(new SelectionListener());
        JScrollPane pBucketList = new JScrollPane(_fBucketList);
        pBucketList.setAlignmentX(Component.LEFT_ALIGNMENT);
        pBucketList.setBorder(UIHelper.dialogGroupBorder());

        _fBucketName = new JTextField(20);
        JPanel pBucketName = new JPanel(new FlowLayout(FlowLayout.LEFT));
        pBucketName.setAlignmentX(Component.LEFT_ALIGNMENT);
        pBucketName.add(new JLabel("Bucket:"));
        pBucketName.add(UIHelper.interButtonSpace());
        pBucketName.add(_fBucketName);

        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.add(new JLabel("Select Existing:"));
        panel.add(pBucketList);
        panel.add(pBucketName);
        return panel;
    }


    private class SelectionListener
    implements ListSelectionListener
    {
        public void valueChanged(ListSelectionEvent evt)
        {
            String selected = (String)_fBucketList.getSelectedValue();
            _fBucketName.setText(selected);
        }
    }


    private class OKAction
    extends AbstractAction
    {
        private static final long serialVersionUID = 1L;

        public OKAction()
        {
            super("Select");
        }

        public void actionPerformed(ActionEvent ignored)
        {
            _dialog.setVisible(false);
            String selected = _fBucketName.getText();
            _concierge.getConfig().setAmazonBucketName(selected);
            _concierge.getMainFrame().resetBucket();

            if (StringUtils.isNotBlank(selected))
                new S3InitialLoadOp(_concierge).start();
        }
    }
}
