// Copyright (c) 2010 Keith D Gregory, all rights reserved
package com.kdgregory.app.s3util.dialogs;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.swinglib.SwingUtil;
import net.sf.swinglib.UIHelper;
import net.sf.swinglib.actions.DialogCloseAction;
import net.sf.swinglib.field.FieldValidator;
import net.sf.swinglib.field.FieldWatcher;
import net.sf.swinglib.layout.CompactGridLayout;

import com.kdgregory.app.s3util.main.Concierge;
import com.kdgregory.app.s3util.main.ConfigBean;


/**
 *  A dialog that allows the user to update their preferences. This is
 *  invoked via action, and will directly update the config bean when
 *  the user clicks Apply.
 */
public class PrefsDialogController
{
    private Concierge _concierge;

    private JDialog _theDialog;
    private JTextField _fEndpoint;
    private JTextField _fPublicKey;
    private JTextField _fPrivateKey;
    private OKAction _bOk;
    private DialogCloseAction _bCancel;

    private FieldWatcher _fieldWatcher;


    public PrefsDialogController(Concierge concierge)
    {
        _concierge = concierge;
    }


    public void show()
    {
        if (_theDialog == null)
            buildDialog();

        ConfigBean prefs = _concierge.getConfig();
        _fEndpoint.setText(prefs.getAmazonEndpoint());
        _fPublicKey.setText(prefs.getAmazonPublicKey());
        _fPrivateKey.setText(prefs.getAmazonPrivateKey());
        _fieldWatcher.reset();

        // since we dispose between calls, re-pack every time
        _theDialog.pack();
        SwingUtil.centerAndShow(_theDialog, _concierge.getDialogOwner());
    }


//----------------------------------------------------------------------------
//  Construction
//----------------------------------------------------------------------------

    private void buildDialog()
    {
        _theDialog = UIHelper.newModalDialog(
                            _concierge.getDialogOwner(),
                            "Preferences",
                            buildContentPane(),
                            buildButtons());
        _bCancel.setDialog(_theDialog);
        configureFieldWatcher();
    }


    private JPanel buildContentPane()
    {
        JPanel content = new JPanel();
        content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
        content.add(new JLabel("Amazon Connection Data"));
        content.add(buildAWSPanel());
        return content;
    }


    private JPanel buildAWSPanel()
    {
        _fEndpoint = new JTextField(20);
        _fEndpoint.setToolTipText(ConfigBean.amazonEndpointInfo());

        _fPublicKey = new JTextField(30);
        _fPublicKey.setToolTipText(ConfigBean.amazonPublicKeyInfo());

        _fPrivateKey = new JTextField(30);
        _fPublicKey.setToolTipText(ConfigBean.amazonPrivateKeyInfo());

        JPanel awsPanel = new JPanel(new CompactGridLayout(2, 12, 6));
        awsPanel.setBorder(UIHelper.dialogGroupBorder());
        awsPanel.add(new JLabel("Endpoint:"));
        awsPanel.add(_fEndpoint);
        awsPanel.add(new JLabel("Public Key:"));
        awsPanel.add(_fPublicKey);
        awsPanel.add(new JLabel("Private Key:"));
        awsPanel.add(_fPrivateKey);
        return awsPanel;
    }


    private Action[] buildButtons()
    {
        _bOk = new OKAction();
        _bCancel = new DialogCloseAction(null, "Cancel", true);
        return new Action[] { _bOk, _bCancel };
    }


    private void configureFieldWatcher()
    {
        _fieldWatcher = new FieldWatcher(_bOk)
        .addValidatedField(
                _fEndpoint,
                new FieldValidator(
                        _fEndpoint,
                        ConfigBean.amazonEndpointRegex(),
                        UIHelper.HIGHLIGHT_COLOR))
        .addValidatedField(
                _fPrivateKey,
                new FieldValidator(
                        _fPrivateKey,
                        ConfigBean.amazonPrivateKeyRegex(),
                        UIHelper.HIGHLIGHT_COLOR))
        .addValidatedField(
                _fPublicKey,
                new FieldValidator(
                        _fPublicKey,
                        ConfigBean.amazonPublicKeyRegex(),
                        UIHelper.HIGHLIGHT_COLOR));
    }


//----------------------------------------------------------------------------
//  Operation
//----------------------------------------------------------------------------

    private class OKAction
    extends AbstractAction
    {
        private static final long serialVersionUID = 1L;

        public OKAction()
        {
            super("OK");
        }

        public void actionPerformed(ActionEvent ignored)
        {
            // note: this dialog will be rarely used; no reason to leave
            //       it around, consuming resources
            _theDialog.dispose();

            ConfigBean prefs = _concierge.getConfig();
            prefs.setAmazonEndpoint(_fEndpoint.getText());
            prefs.setAmazonPublicKey(_fPublicKey.getText());
            prefs.setAmazonPrivateKey(_fPrivateKey.getText());
        }
    }
}
