// Copyright (c) 2010 Keith D Gregory, all rights reserved
package com.kdgregory.app.s3util.main;

import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;

import org.apache.commons.lang.StringUtils;

import net.sf.swinglib.SwingUtil;
import net.sf.swinglib.components.MainFrame;
import net.sf.swinglib.listeners.PopupListener;
import net.sf.swinglib.table.FormattingRenderer;
import net.sf.swinglib.table.TableUtil;

import com.kdgregory.app.s3util.filelist.FileListTableModel;
import com.kdgregory.app.s3util.filelist.S3File;


/**
 *  Responsible for building out the application main frame. This class may
 *  be constructed anywhere, but {@link #buildAndShow} must be called from
 *  the event thread.
 */
public class MainFrameController
{
    private final static String BASE_TITLE = "S3Util";

    private Concierge _concierge;
    private ActionRegistry _actionRegistry;

    private JFrame _mainFrame;
    private JTable _table;
    private FileListTableModel _fileList;
    private DefaultTableModel _emptyList;


    public MainFrameController(Concierge concierge)
    {
        _concierge = concierge;
        _actionRegistry = new ActionRegistry(_concierge);
    }


//----------------------------------------------------------------------------
//  Initialization
//----------------------------------------------------------------------------

    public void buildAndShow()
    {
        _mainFrame = new MainFrame(BASE_TITLE, _actionRegistry.fileQuit);
        _mainFrame.setJMenuBar(createMainMenu());
        _mainFrame.setContentPane(createContentPane());
        _mainFrame.pack();
        _concierge.setMainFrame(this, _mainFrame);

        PopupListener.attach(_table, createPopupMenu());
        resetBucket();

        SwingUtil.centerAndShow(_mainFrame);
    }


    private JMenuBar createMainMenu()
    {
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);
        fileMenu.add(_actionRegistry.fileRefresh);
        fileMenu.add(new JSeparator(JSeparator.HORIZONTAL));
        fileMenu.add(_actionRegistry.fileUpload);
        fileMenu.add(_actionRegistry.fileDownload);
        fileMenu.add(_actionRegistry.fileDelete);
        fileMenu.add(new JSeparator(JSeparator.HORIZONTAL));
        fileMenu.add(_actionRegistry.fileSetPrefs);
        fileMenu.add(new JSeparator(JSeparator.HORIZONTAL));
        fileMenu.add(_actionRegistry.fileQuit);

        JMenu editMenu = new JMenu("Edit");
        fileMenu.setMnemonic(KeyEvent.VK_E);
        editMenu.add(_actionRegistry.editSelectAll);
        editMenu.add(_actionRegistry.editSelectNone);
        editMenu.add(new JSeparator(JSeparator.HORIZONTAL));
        editMenu.add(_actionRegistry.editCopyUrl);

        JMenu bucketMenu = new JMenu("Bucket");
        fileMenu.setMnemonic(KeyEvent.VK_B);
        bucketMenu.add(_actionRegistry.bucketSelect);
        bucketMenu.add(new JSeparator(JSeparator.HORIZONTAL));
        bucketMenu.add(_actionRegistry.bucketDelete);

        JMenuBar menuBar = new JMenuBar();
        menuBar.add(fileMenu);
        menuBar.add(editMenu);
        menuBar.add(bucketMenu);
        return menuBar;
    }


    private JPopupMenu createPopupMenu()
    {
        JPopupMenu menu = new JPopupMenu();
        menu.add(_actionRegistry.editSelectAll);
        menu.add(_actionRegistry.editSelectNone);
        menu.add(new JSeparator(JSeparator.HORIZONTAL));
        menu.add(_actionRegistry.editCopyUrl);
        menu.add(new JSeparator(JSeparator.HORIZONTAL));
        menu.add(_actionRegistry.fileUpload);
        menu.add(_actionRegistry.fileDownload);
        menu.add(_actionRegistry.fileDelete);
        menu.add(new JSeparator(JSeparator.HORIZONTAL));
        menu.add(_actionRegistry.fileRefresh);
        return menu;
    }


    private JScrollPane createContentPane()
    {
        _emptyList = new DefaultTableModel(
                        new Object[0][0],
                        new String[] { "Loading..." });

        _table = new JTable();
        _table.setFocusable(false);
        _table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        _table.setColumnSelectionAllowed(false);
        _table.getSelectionModel().addListSelectionListener(new SelectionListener());
        _table.setDefaultRenderer(
                Integer.class,
                new FormattingRenderer(new DecimalFormat("#,##0")));
        _table.setDefaultRenderer(
                Date.class,
                new FormattingRenderer(new SimpleDateFormat("MMM dd yyyy HH:mm:ss")));

        JScrollPane container = new JScrollPane(_table);
        container.setPreferredSize(new Dimension(600, 400));
        return container;
    }


//----------------------------------------------------------------------------
//  Operational Methods
//----------------------------------------------------------------------------

    /**
     *  Instructs the frame to indicate the status of a background operation.
     *
     *  @param  isBusy  Pass <code>true</code> to indicate that an operation
     *                  is in progress, <code>false</code> to indicate that
     *                  it has completed.
     */
    public void setBusyState(boolean isBusy)
    {
        if (isBusy)
            _concierge.getCursorManager()
                      .pushBusyCursor(_mainFrame.getRootPane());
        else
            _concierge.getCursorManager()
                      .popCursor(_mainFrame.getRootPane());
    }


    /**
     *  Updates the frame and actions to match the current bucket,
     *  if any.
     */
    public void resetBucket()
    {
        String bucketName = _concierge.getConfig().getAmazonBucketName();
        if (StringUtils.isEmpty(bucketName))
        {
            // the only time we'll have an empty bucket is when the
            // program is first initialized, or for the short time
            // between deleting a bucket and selecting a new one
            _mainFrame.setTitle(BASE_TITLE);
            _actionRegistry.updatePerActiveBucket(false);
        }
        else
        {
            _mainFrame.setTitle(BASE_TITLE + " - " + bucketName);
            _actionRegistry.updatePerActiveBucket(true);
        }

        // regardless of whether we've got a bucket or not, we will reset
        // the displayed list
        _table.setModel(_emptyList);
        _fileList = new FileListTableModel();
    }


    /**
     *  Resets the entire list of files.
     */
    public void resetList(FileListTableModel model)
    {
        _fileList = model;
        displayActualList();
        selectNone();
    }


    /**
     *  Adds a single file to the list (will update an existing file with
     *  the same key).
     */
    public void addFile(S3File file)
    {
        _fileList.addFile(file);
    }


    /**
     *  Deletes a single file from the  list. The scroll is not changed.
     */
    public void deleteFile(S3File file)
    {
        _fileList.deleteFile(file);
    }


    /**
     *  Returns the number of files selected.
     */
    public int getSelectionCount()
    {
        return _table.getSelectedRowCount();
    }


    /**
     *  Returns the list of currently selected files.
     */
    public List<S3File> getSelectedFiles()
    {
        int[] selection = _table.getSelectedRows();
        List<S3File> result = new ArrayList<S3File>(selection.length);
        for (int row : selection)
            result.add(_fileList.getFileAt(row));
        return result;
    }


    /**
     *  Sets the "selected" flag on all current files.
     */
    public void selectAll()
    {
        _table.selectAll();
    }


    /**
     *  Clears the "selected" flag on all current files.
     */
    public void selectNone()
    {
        _table.clearSelection();
    }


//----------------------------------------------------------------------------
//  Internals
//----------------------------------------------------------------------------

    /**
     *  A helper method that determines whether the table is currently
     *  displaying the dummy "Loading" list, or the real list. If the
     *  former, switches list models and properly sets column sizes.
     */
    private void displayActualList()
    {
        if (_table.getModel() != _fileList)
        {
            _table.setModel(_fileList);
            TableUtil.setRelativeColumnWidths(
                    _table,
                    _fileList.getRelativeWidths());
        }
    }


    /**
     *  Listens for changes to the selection, and enables/disables actions
     *  appropriately.
     */
    private class SelectionListener
    implements ListSelectionListener
    {
        public void valueChanged(ListSelectionEvent evt)
        {
            _actionRegistry.updatePerSelection(getSelectionCount());
        }
    }
}
